use std::io::Read;
use std::process::{Child, Command, Stdio};

use crate::app::App;
use crate::result::Result;

pub struct Proc {
	pub app: App,
	pub child: Child,
}

impl Proc {
	pub fn start(app: App) -> Result<Self> {
		info!("starting application {}", &app.name);

		let mut command = Command::new(&app.command);
		command.stdout(Stdio::piped());
		command.stderr(Stdio::piped());
		command.args(&app.args);

		let child = command.spawn()?;

		Ok(Proc { app, child })
	}

	pub fn check_stdout(&mut self) -> Option<String> {
		let child_stdout = self.child.stdout.as_mut();
		if let Some(stdout) = child_stdout {
			let mut buf = String::new();
			match stdout.read_to_string(&mut buf) {
				Ok(size) => {
					if size > 0 {
						return Some(buf);
					} else {
						return None;
					}
				}
				Err(e) => {
					error!("couldn't read stdout from {}: {:?}", &self.app.name, e);
					return None;
				}
			}
		}
		None
	}

	pub fn check_stderr(&mut self) -> Option<String> {
		let child_stderr = self.child.stderr.as_mut();
		if let Some(stderr) = child_stderr {
			let mut buf = String::new();
			match stderr.read_to_string(&mut buf) {
				Ok(size) => {
					if size > 0 {
						return Some(buf);
					} else {
						return None;
					}
				}
				Err(e) => {
					error!("couldn't read stderr from {}: {:?}", &self.app.name, e);
					return None;
				}
			}
		}
		None
	}
}
